@@ -21,6 +21,10 @@ class iriref(str):
     'spam'
     >>> iriref('spam eggs')
     [raises ValueError]
+
+    >>> from versa import I
+    >>> I('spam')
+    'spam'
     '''
     def __new__(cls, value):
         if not iri.matches_uri_ref_syntax(value):
@@ -32,3 +36,15 @@ class iriref(str):
 
     def __repr__(self):
         return u'I(' + str(self) + ')'
+
+    def __call__(self, tail):
+        '''
+        >>> from versa import I
+        >>> base = I('https://example.org/')
+        >>> a = base('a')
+        >>> a
+        I(https://example.org/a)
+        '''
+        # Just dumb concatenation for now
+        return iriref(str(self) + str(tail))
+
